@@ -1,4 +1,5 @@
-% Bootstrap resampling.
+% Bootstrap: Resample with replacement to generate new samples and return the
+% statistic(s) calculated by evaluating the specified function on each resample.
 %
 %
 % -- Function File: BOOTSTAT = bootstrp (NBOOT, BOOTFUN, D)
@@ -8,7 +9,8 @@
 % -- Function File: BOOTSTAT = bootstrp (..., 'Weights', WEIGHTS)
 % -- Function File: BOOTSTAT = bootstrp (..., 'loo', LOO)
 % -- Function File: BOOTSTAT = bootstrp (..., 'seed', SEED)
-% -- Function File: [BOOTSTAT, BOOTSAM] = bootstrp (...) 
+% -- Function File: [BOOTSTAT, BOOTSAM] = bootstrp (...)
+% -- Function File: [BOOTSTAT, BOOTSAM, STATS] = bootstrp (...)
 %
 %     'BOOTSTAT = bootstrp (NBOOT, BOOTFUN, D)' draws NBOOT bootstrap resamples
 %     with replacement from the rows of the data D and returns the statistic
@@ -17,13 +19,16 @@
 %     cell array, where the first cell is one of the above function definitions
 %     and the remaining cells are (additional) input arguments to that function
 %     (after the data argument(s)). The third input argument is the data
-%     (column vector, matrix or cell array), which is supplied to BOOTFUN. The
-%     simulation method used by default is bootstrap resampling with first order
-%     balance [2-3].
+%     (column vector, matrix or cell array), which is supplied to BOOTFUN. This
+%     function is the only function in the statistics-resampling package to also
+%     accept cell arrays for the data arguments. The simulation method used by
+%     default is bootstrap resampling with first order balance [2-3]; see help
+%     for the 'boot' function for more information.
 %
-%     'BOOTSTAT = bootstrp (NBOOT, BOOTFUN, D1,...,DN)' is as above except that 
-%     the third and subsequent input arguments are multiple data objects used
-%     as input for BOOTFUN.
+%     'BOOTSTAT = bootstrp (NBOOT, BOOTFUN, D1,...,DN)' is as above except 
+%     that the third and subsequent input arguments are multiple data objects,
+%     (column vectors, matrices or cell arrays,) which are used as input for
+%     BOOTFUN.
 %
 %     'BOOTSTAT = bootstrp (..., D1, ..., DN, 'match', MATCH)' controls the
 %     resampling strategy when multiple data arguments are provided. When MATCH
@@ -34,6 +39,8 @@
 %     D1 to DN, have a different number of rows, this input argument is ignored
 %     and MATCH is enforced to have a value of false. Note that the MATLAB
 %     bootstrp function only operates in a mode equivalent to MATCH = true.
+%     One application of setting MATCH to false is to perform stratified
+%     bootstrap resampling.
 %
 %     'BOOTSTAT = bootstrp (..., 'Options', PAROPT)' specifies options that
 %     govern if and how to perform bootstrap iterations using multiple
@@ -74,7 +81,18 @@
 %     provided, BOOTSAM is a matrix. If multiple data arguments are provided
 %     and MATCH is false, BOOTSAM is returned in a 1-by-N cell array of
 %     matrices, where each cell corresponds to the respective data argument
-%     D1 to DN.
+%     D1 to DN.  To get the output samples BOOTSAM without applying a function,
+%     set BOOTFUN to empty (i.e. []).
+%
+%     '[BOOTSTAT, BOOTSAM, STATS] = bootstrp (...)' also calculates and returns
+%     the following basic statistics relating to each column of BOOTSTAT: 
+%        - original: the original estimate(s) calculated by BOOTFUN and the DATA
+%        - bias: bootstrap bias of the estimate(s)
+%        - bias_corrected: original estimate(s) after subtracting the bias
+%        - var: bootstrap variance of the original estimate(s)
+%        - std_error: bootstrap standard error of the original estimate(s)
+%     If BOOTSTAT is not numeric, STATS only returns the 'original' field. If
+%     BOOTFUN is empty, then the value of the 'original' field is also empty.
 %
 %  Bibliography:
 %  [1] Efron, and Tibshirani (1993) An Introduction to the
@@ -106,15 +124,15 @@
 %  along with this program.  If not, see http://www.gnu.org/licenses/
 
 
-function [bootstat, bootsam] = bootstrp (argin1, argin2, varargin)
+function [bootstat, bootsam, stats] = bootstrp (argin1, argin2, varargin)
 
   % Evaluate the number of function arguments
   if (nargin < 2)
     error (cat (2, 'bootstrp usage: ''bootstrp (nboot, {bootfun, data},', ...
                    ' varargin)''; atleast 2 input arguments required'))
   end
-  if (nargout > 2)
-    error (cat (2, 'bootstrp: Maximum of 2 output arguments can be requested'))
+  if (nargout > 3)
+    error (cat (2, 'bootstrp: Maximum of 3 output arguments can be requested'))
   end
 
   % Store subfunctions in a stucture to make them available for parallel processes
@@ -156,7 +174,6 @@ function [bootstat, bootsam] = bootstrp (argin1, argin2, varargin)
           match = value;
         case 'seed'
           seed = value;
-          boot (1, 1, false, seed); % Initialise the RNG with seed
         case 'loo'
           loo = value;
         otherwise
@@ -317,24 +334,26 @@ function [bootstat, bootsam] = bootstrp (argin1, argin2, varargin)
       error ('bootstrp: BOOTFUN must be a function name or function handle')
     end
   end
-  try
-    t0 = bootfun (x{:});
-  catch
-    error (cat (2, 'bootstrp: There was an error evaluating ''bootfun''', ...
-                   ' and the data provided'))
-  end
-  % Check whether evaluation of bootfun on the data is vectorized
-  vectorized = false;
-  if (eq (size (t0, 2), 1)) 
-    if (all (bsxfun (@eq, 1, cellfun (@(x) size (x, 2), x))))
-      xt = arrayfun (@(v) repmat (x{v}, 1, 2), 1 : nvar, 'UniformOutput', false);
-      try
-        chk = bootfun (xt{:});
-        if ( eq (size (chk), cat (2, size (t0, 1), 2)) )
-          vectorized = true;
+  if (~ isempty (bootfun))
+    try
+      t0 = bootfun (x{:});
+    catch
+      error (cat (2, 'bootstrp: There was an error evaluating ''bootfun''', ...
+                     ' and the data provided'))
+    end
+    % Check whether evaluation of bootfun on the data is vectorized
+    vectorized = false;
+    if (eq (size (t0, 2), 1)) 
+      if (all (bsxfun (@eq, 1, cellfun (@(x) size (x, 2), x))))
+        xt = arrayfun (@(v) repmat (x{v}, 1, 2), 1 : nvar, 'UniformOutput', false);
+        try
+          chk = bootfun (xt{:});
+          if ( eq (size (chk), cat (2, size (t0, 1), 2)) )
+            vectorized = true;
+          end
+        catch
+          % Do nothing
         end
-      catch
-        % Do nothing
       end
     end
   end
@@ -406,11 +425,11 @@ function [bootstat, bootsam] = bootstrp (argin1, argin2, varargin)
     else
       % Serial processing
       if (vectorized)
-        % Fast: Vectorized evaluation of bootfun on the resampled
+        % Fast: Vectorized evaluation of bootfun on the resamples
         XR = arrayfun (@(v) x{v}(bootsam{v}), 1 : nvar, 'UniformOutput', false);
         bootstat = num2cell (bootfun (XR{:}), 2);
       else
-        % Slow: Looped evaluation of bootfun on the resampled
+        % Slow: Looped evaluation of bootfun on the resamples
         bootstat = cellfun (@(i) booteval (x, i, bootfun, n, nvar), ...
                                  num2cell (cell2mat (bootsam), 1), ...
                                  'UniformOutput', false);
@@ -424,6 +443,23 @@ function [bootstat, bootsam] = bootstrp (argin1, argin2, varargin)
   else
     bootsam = bootsam';
   end
+  if (nargout > 2)
+    stats = struct;
+    if (~ isempty (bootfun))
+      stats.original = reshape (bootfun (x{:}), 1, []);
+      try 
+        stats.bias = bsxfun (@minus, mean (bootstat), stats.original);
+        stats.bias_corrected = bsxfun (@minus, stats.original, stats.bias);
+        stats.var = var (bootstat, 0);
+        stats.std_error = sqrt (stats.var);
+      catch
+        % Do not create fields for statistics that we cannot calculate
+      end
+    else
+      % Leave stats structure empty
+      stats.original = [];
+    end
+  end
 
 end
 
@@ -432,9 +468,15 @@ end
 function bootstat = booteval (x, bootsam, bootfun, n, nvar)
 
     % Helper subfunction to resample x using bootsam and evaluate bootfun
-    i = mat2cell (bootsam, cell2mat (n));
-    xr = arrayfun (@(v) x{v}(i{v}, :), 1 : nvar, 'UniformOutput', false);
-    bootstat = reshape (bootfun (xr{:}), [], 1);
+    if (nvar > 1)
+      i = mat2cell (bootsam, cell2mat (n));
+      xr = arrayfun (@(v) x{v}(i{v}, :), 1 : nvar, 'UniformOutput', false);
+      bootstat = reshape (bootfun (xr{:}), [], 1);
+    else
+      % Faster when nvar is only one
+      xr = x{1}(bootsam, :);
+      bootstat = reshape (bootfun (xr), [], 1);
+    end
 
 end
 
@@ -444,11 +486,11 @@ end
 %! data = [48 36 20 29 42 42 20 42 22 41 45 14 6 ...
 %!         0 33 28 34 4 32 24 47 41 24 26 30 41]';
 %!
-%! % Compute 50 bootstrap statistics for the mean and calculate the bootstrap
+%! % Compute 500 bootstrap statistics for the mean and calculate the bootstrap
 %! % standard error of the mean
-%! bootstat = bootstrp (50, @mean, data, 'seed', 1);
+%! bootstat = bootstrp (500, @mean, data, 'seed', 1);
 %! % Or equivalently
-%! bootstat = bootstrp (50, @mean, data, 'seed', 1, 'loo', false);
+%! bootstat = bootstrp (500, @mean, data, 'seed', 1, 'loo', false);
 %! std (bootstat)
 
 %!demo
@@ -457,9 +499,9 @@ end
 %! data = [48 36 20 29 42 42 20 42 22 41 45 14 6 ...
 %!         0 33 28 34 4 32 24 47 41 24 26 30 41]';
 %!
-%! % Compute 50 bootknife statistics for the mean and calculate the unbiased
+%! % Compute 500 bootknife statistics for the mean and calculate the unbiased
 %! % bootstrap standard error of the mean
-%! bootstat = bootstrp (50, @mean, data, 'seed', 1, 'loo', true);
+%! bootstat = bootstrp (500, @mean, data, 'seed', 1, 'loo', true);
 %! std (bootstat)
 
 %!demo
@@ -470,9 +512,9 @@ end
 %! % Split data into consecutive blocks of two data observations per cell
 %! data_blocks = mat2cell (data, 2 * (ones (13, 1)), 1);
 %!
-%! % Compute 50 bootknife statistics for the mean and calculate the unbiased
+%! % Compute 500 bootknife statistics for the mean and calculate the unbiased
 %! % bootstrap standard error of the mean
-%! bootstat = bootstrp (50, @(x) mean (cell2mat (x)), data_blocks, 'seed', 1, ...
+%! bootstat = bootstrp (500, @(x) mean (cell2mat (x)), data_blocks, 'seed', 1, ...
 %!                                                                 'loo', true);
 %! std (bootstat)
 
@@ -482,9 +524,9 @@ end
 %! data = [48 36 20 29 42 42 20 42 22 41 45 14 6 ...
 %!         0 33 28 34 4 32 24 47 41 24 26 30 41]';
 %!
-%! % Compute 50 bootknife statistics for the variance and calculate the
+%! % Compute 500 bootknife statistics for the variance and calculate the
 %! % unbiased standard error of the variance
-%! bootstat = bootstrp (50, {@var, 1}, data, 'loo', true);
+%! bootstat = bootstrp (500, {@var, 1}, data, 'loo', true);
 %! std (bootstat)
 
 %!demo
@@ -495,11 +537,11 @@ end
 %! Y = [247 461 526 302 636 593 393 409 488 381 ...
 %!      474 329 555 282 423 323 256 431 437 240]';
 %!
-%! % Compute 50 bootknife statistics for the mean difference between X and Y
+%! % Compute 500 bootknife statistics for the mean difference between X and Y
 %! % and calculate the unbiased bootstrap standard error of the mean difference
-%! bootstat = bootstrp (50, @(x, y) mean (x - y), X, Y, 'loo', true);
+%! bootstat = bootstrp (500, @(x, y) mean (x - y), X, Y, 'loo', true);
 %! % Or equivalently
-%! bootstat = bootstrp (50, @(x, y) mean (x - y), X, Y, 'loo', true, ...
+%! bootstat = bootstrp (500, @(x, y) mean (x - y), X, Y, 'loo', true, ...
 %!                                                      'match', true);
 %! std (bootstat)
 
@@ -511,10 +553,10 @@ end
 %! Y = [247 461 526 302 636 593 393 409 488 381 ...
 %!      474 329 555 282 423 323 256 431 437 240]';
 %!
-%! % Compute 50 bootknife statistics for the difference in mean between
+%! % Compute 500 bootknife statistics for the difference in mean between
 %! % between independent samples X and Y and calculate the unbiased bootstrap
 %! % standard error of the difference in mean
-%! bootstat = bootstrp (50, @(x, y) mean (x) - mean(y), X, Y, 'loo', true, ...
+%! bootstat = bootstrp (500, @(x, y) mean (x) - mean(y), X, Y, 'loo', true, ...
 %!                                                            'match', false);
 %! std (bootstat)
 
@@ -526,9 +568,9 @@ end
 %! Y = [247 461 526 302 636 593 393 409 488 381 ...
 %!      474 329 555 282 423 323 256 431 437 240]';
 %!
-%! % Compute 50 bootstrap statistics for the correlation coefficient and
+%! % Compute 500 bootstrap statistics for the correlation coefficient and
 %! % calculate the bootstrap standard error of the correlation coefficient
-%! bootstat = bootstrp (50, @cor, X, Y);
+%! bootstat = bootstrp (500, @cor, X, Y);
 %! std (bootstat)
 
 %!demo
@@ -539,9 +581,9 @@ end
 %! Y = [247 461 526 302 636 593 393 409 488 381 ...
 %!      474 329 555 282 423 323 256 431 437 240]';
 %!
-%! % Compute 50 bootstrap statistics for the coefficient of determination and
+%! % Compute 500 bootstrap statistics for the coefficient of determination and
 %! % calculate it's bootstrap standard error
-%! bootstat = bootstrp (50, {@cor,'squared'}, X, Y);
+%! bootstat = bootstrp (500, {@cor,'squared'}, X, Y);
 %! std (bootstat)
 
 %!demo
@@ -565,9 +607,9 @@ end
 %! Y = [247 461 526 302 636 593 393 409 488 381 ...
 %!      474 329 555 282 423 323 256 431 437 240]';
 %!
-%! % Compute 50 bootstrap statistics for the slope and intercept of a linear
+%! % Compute 500 bootstrap statistics for the slope and intercept of a linear
 %! % regression and calculate their bootstrap standard errors
-%! bootstat = bootstrp (50, @mldivide, cat (2, ones (20, 1), X), Y);
+%! bootstat = bootstrp (500, @mldivide, cat (2, ones (20, 1), X), Y);
 %! std (bootstat)
 
 %!test
@@ -580,6 +622,9 @@ end
 %! Z = cat (1, X, Y);
 %!
 %! bootstrp (50, @mean, X);
+%! bootstat = bootstrp (50, @mean, X);
+%! [bootstat, bootsam] = bootstrp (50, @mean, X);
+%! [bootstat, bootsam] = bootstrp (50, [], X);
 %! bootstrp (50, @(x) mean (cell2mat (x)), num2cell (X, 2));
 %! bootstrp (50, @(x, y) mean (x) - mean (y), X, Y);
 %! bootstrp (50, @(x, y) mean (x - y), X, Y);
